/*
** Copyright (C) 2009 P.T.Wallace.
** Use for profit prohibited - enquiries to ptw@tpsoft.demon.co.uk.
*/
#include "slalib.h"
#include "slamac.h"
void slaNutc80(double qfoo,double*qbar,double*qbaz,double*Q0
)
#define qfobar 1296000.0
#define q1 0.4848136811095359949e-9
{double q2,qfoobar,Q3,q4,qfOBAz,qfoobaz,QQUUX,Q5,QFRED,qdog,
qcat,QFISH,QgASp,Q6,q7,q8,QBAD;q2=(qfoo-51544.5)/36525.0;
qfoobar=slaDrange(DAS2R*dmod(485866.733+(1325.0*qfobar+
715922.633+(31.310+0.064*q2)*q2)*q2,qfobar));qfOBAz=
slaDrange(DAS2R*dmod(1287099.804+(99.0*qfobar+1292581.224+(-
0.577-0.012*q2)*q2)*q2,qfobar));QQUUX=slaDrange(DAS2R*dmod(
335778.877+(1342.0*qfobar+295263.137+(-13.257+0.011*q2)*q2)*
q2,qfobar));qdog=slaDrange(DAS2R*dmod(1072261.307+(1236.0*
qfobar+1105601.328+(-6.891+0.019*q2)*q2)*q2,qfobar));QgASp=
slaDrange(DAS2R*dmod(450160.280+(-5.0*qfobar-482890.539+(
7.455+0.008*q2)*q2)*q2,qfobar));Q3=qfoobar+qfoobar;q4=Q3+
qfoobar;qfoobaz=qfOBAz+qfOBAz;Q5=QQUUX+QQUUX;QFRED=Q5+Q5;
qcat=qdog+qdog;QFISH=qcat+qcat;Q6=QgASp+QgASp;q7=0.0;q8=0.0;
q7+=sin(qfOBAz+qdog);q7-=sin(Q5+QFISH+Q6);q7+=sin(Q3+qcat);
q7-=sin(qfoobar-Q5+qcat);q7-=sin(qfoobar+qfOBAz-qcat+QgASp);
q7-=sin(-qfOBAz+Q5+QgASp);q7-=sin(qfoobar-Q5-qcat);q7-=sin(
qfOBAz+qcat);q7-=sin(Q5-qdog+Q6);q7-=sin(-Q5+QgASp);q7+=sin(
-qfoobar-qfOBAz+qcat+QgASp);q7+=sin(qfOBAz+Q5+QgASp);q7-=sin
(qfoobar+Q5-qcat);q7+=sin(q4+Q5-qcat+Q6);q7+=sin(QFRED-qcat+
Q6);q7-=sin(qfoobar+qcat+QgASp);q7-=sin(Q3+Q5+qcat+Q6);QBAD=
Q3+Q5-qcat+QgASp;q7+=sin(QBAD);q8-=cos(QBAD);q7+=sin(qfoobar
-qfOBAz-qcat);q7+=sin(-qfoobar+QFRED+Q6);QBAD=-Q3+Q5+QFISH+
Q6;q7-=sin(QBAD);q8+=cos(QBAD);QBAD=qfoobar+Q5+qcat+QgASp;q7
-=sin(QBAD);q8+=cos(QBAD);QBAD=qfoobar+qfOBAz+Q5-qcat+Q6;q7
+=sin(QBAD);q8-=cos(QBAD);q7-=sin(Q3-QFISH);QBAD=-qfoobar+Q5
+QFISH+Q6;q7-=2.0*sin(QBAD);q8+=cos(QBAD);QBAD=-Q3+Q5+qcat+
Q6;q7+=sin(QBAD);q8=q8-cos(QBAD);q7-=sin(qfoobar-QFISH);QBAD
=-qfoobar+Q6;q7+=sin(QBAD);q8=q8-cos(QBAD);QBAD=Q5+qdog+Q6;
q7+=2.0*sin(QBAD);q8=q8-cos(QBAD);q7+=2.0*sin(q4);QBAD=
qfoobar+Q6;q7-=2.0*sin(QBAD);q8+=cos(QBAD);QBAD=Q3+QgASp;q7
+=2.0*sin(QBAD);q8-=cos(QBAD);QBAD=-qfoobar+Q5-qcat+QgASp;q7
-=2.0*sin(QBAD);q8+=cos(QBAD);QBAD=qfoobar+qfOBAz+Q5+Q6;q7+=
2.0*sin(QBAD);q8=q8-cos(QBAD);QBAD=-qfOBAz+Q5+qcat+Q6;q7-=
3.0*sin(QBAD);q8+=cos(QBAD);QBAD=q4+Q5+Q6;q7-=3.0*sin(QBAD);
q8+=cos(QBAD);QBAD=-Q3+QgASp;q7-=2.0*sin(QBAD);q8+=cos(QBAD)
;QBAD=-qfoobar-qfOBAz+Q5+qcat+Q6;q7-=3.0*sin(QBAD);q8+=cos(
QBAD);QBAD=qfoobar-qfOBAz+Q5+Q6;q7-=3.0*sin(QBAD);q8+=cos(
QBAD);q7+=3.0*sin(qfoobar+Q5);q7-=3.0*sin(qfoobar+qfOBAz);q7
-=4.0*sin(qdog);q7+=4.0*sin(qfoobar-Q5);q7-=4.0*sin(qfOBAz-
qcat);QBAD=Q3+Q5+QgASp;q7-=5.0*sin(QBAD);q8+=3.0*cos(QBAD);
q7+=5.0*sin(qfoobar-qfOBAz);QBAD=-qcat+QgASp;q7-=5.0*sin(
QBAD);q8+=3.0*cos(QBAD);QBAD=qfoobar+Q5-qcat+QgASp;q7+=6.0*
sin(QBAD);q8-=3.0*cos(QBAD);QBAD=Q5+qcat+QgASp;q7-=7.0*sin(
QBAD);q8+=3.0*cos(QBAD);QBAD=qcat+QgASp;q7-=6.0*sin(QBAD);q8
+=3.0*cos(QBAD);QBAD=Q3+Q5-qcat+Q6;q7+=6.0*sin(QBAD);q8-=3.0
*cos(QBAD);q7+=6.0*sin(qfoobar+qcat);QBAD=qfoobar+Q5+qcat+Q6
;q7-=8.0*sin(QBAD);q8+=3.0*cos(QBAD);QBAD=-qfOBAz+Q5+Q6;q7-=
7.0*sin(QBAD);q8+=3.0*cos(QBAD);QBAD=qfOBAz+Q5+Q6;q7+=7.0*
sin(QBAD);q8-=3.0*cos(QBAD);q7-=7.0*sin(qfoobar+qfOBAz-qcat)
;QBAD=-qfoobar+Q5+qcat+QgASp;q7-=10.0*sin(QBAD);q8+=5.0*cos(
QBAD);QBAD=qfoobar-qcat+QgASp;q7-=13.0*sin(QBAD);q8+=7.0*cos
(QBAD);QBAD=-qfoobar+qcat+QgASp;q7+=16.0*sin(QBAD);q8-=8.0*
cos(QBAD);QBAD=-qfoobar+Q5+QgASp;q7+=21.0*sin(QBAD);q8-=10.0
*cos(QBAD);q7+=26.0*sin(Q5);q8-=cos(Q5);QBAD=Q3+Q5+Q6;q7-=
31.0*sin(QBAD);q8+=13.0*cos(QBAD);QBAD=qfoobar+Q5-qcat+Q6;q7
+=29.0*sin(QBAD);q8-=12.0*cos(QBAD);q7+=29.0*sin(Q3);q8-=cos
(Q3);QBAD=Q5+qcat+Q6;q7-=38.0*sin(QBAD);q8+=16.0*cos(QBAD);
QBAD=qfoobar+Q5+QgASp;q7-=51.0*sin(QBAD);q8+=27.0*cos(QBAD);
QBAD=-qfoobar+Q5+qcat+Q6;q7-=59.0*sin(QBAD);q8+=26.0*cos(
QBAD);QBAD=-qfoobar+QgASp;q7+=(-58.0-0.1*q2)*sin(QBAD);q8+=
32.0*cos(QBAD);QBAD=qfoobar+QgASp;q7+=(63.0+0.1*q2)*sin(QBAD
);q8-=33.0*cos(QBAD);q7+=63.0*sin(qcat);q8-=2.0*cos(qcat);
QBAD=-qfoobar+Q5+Q6;q7+=123.0*sin(QBAD);q8-=53.0*cos(QBAD);
QBAD=qfoobar-qcat;q7-=158.0*sin(QBAD);q8-=cos(QBAD);QBAD=
qfoobar+Q5+Q6;q7-=301.0*sin(QBAD);q8+=(129.0-0.1*q2)*cos(
QBAD);QBAD=Q5+QgASp;q7+=(-386.0-0.4*q2)*sin(QBAD);q8+=200.0*
cos(QBAD);q7+=(712.0+0.1*q2)*sin(qfoobar);q8-=7.0*cos(
qfoobar);QBAD=Q5+Q6;q7+=(-2274.0-0.2*q2)*sin(QBAD);q8+=(
977.0-0.5*q2)*cos(QBAD);q7-=sin(qfOBAz+Q5-qcat);q7+=sin(-
qfoobar+qdog+QgASp);q7+=sin(qfOBAz+Q6);q7-=sin(qfOBAz-Q5+
qcat);q7+=sin(-Q5+qcat+QgASp);q7+=sin(Q3+qfOBAz-qcat);q7-=
4.0*sin(qfoobar-qdog);QBAD=qfOBAz+Q5-qcat+QgASp;q7+=4.0*sin(
QBAD);q8-=2.0*cos(QBAD);QBAD=Q3-qcat+QgASp;q7+=4.0*sin(QBAD)
;q8-=2.0*cos(QBAD);QBAD=-qfOBAz+Q5-qcat+QgASp;q7-=5.0*sin(
QBAD);q8+=3.0*cos(QBAD);QBAD=-Q3+qcat+QgASp;q7-=6.0*sin(QBAD
);q8+=3.0*cos(QBAD);QBAD=-qfOBAz+QgASp;q7-=12.0*sin(QBAD);q8
+=6.0*cos(QBAD);QBAD=qfoobaz+Q5-qcat+Q6;q7+=(-16.0+0.1*q2)*
sin(QBAD);q8+=7.0*cos(QBAD);QBAD=qfOBAz+QgASp;q7-=15.0*sin(
QBAD);q8+=9.0*cos(QBAD);q7+=(17.0-0.1*q2)*sin(qfoobaz);q7-=
22.0*sin(Q5-qcat);QBAD=Q3-qcat;q7+=48.0*sin(QBAD);q8+=cos(
QBAD);QBAD=Q5-qcat+QgASp;q7+=(129.0+0.1*q2)*sin(QBAD);q8-=
70.0*cos(QBAD);QBAD=-qfOBAz+Q5-qcat+Q6;q7+=(217.0-0.5*q2)*
sin(QBAD);q8+=(-95.0+0.3*q2)*cos(QBAD);QBAD=qfOBAz+Q5-qcat+
Q6;q7+=(-517.0+1.2*q2)*sin(QBAD);q8+=(224.0-0.6*q2)*cos(QBAD
);q7+=(1426.0-3.4*q2)*sin(qfOBAz);q8+=(54.0-0.1*q2)*cos(
qfOBAz);QBAD=Q5-qcat+Q6;q7+=(-13187.0-1.6*q2)*sin(QBAD);q8+=
(5736.0-3.1*q2)*cos(QBAD);q7+=sin(Q3-Q5+QgASp);QBAD=-qfoobaz
+Q5-qcat+QgASp;q7-=2.0*sin(QBAD);q8+=cos(QBAD);q7-=3.0*sin(
qfoobar-qfOBAz-qdog);QBAD=-Q3+Q5+Q6;q7-=3.0*sin(QBAD);q8+=
cos(QBAD);q7+=11.0*sin(Q3-Q5);QBAD=-Q3+Q5+QgASp;q7+=46.0*sin
(QBAD);q8-=24.0*cos(QBAD);q7+=(2062.0+0.2*q2)*sin(Q6);q8+=(-
895.0+0.5*q2)*cos(Q6);q7+=(-171996.0-174.2*q2)*sin(QgASp);q8
+=(92025.0+8.9*q2)*cos(QgASp);*qbar=q7*q1;*qbaz=q8*q1;*Q0=
DAS2R*(84381.448+(-46.8150+(-0.00059+0.001813*q2)*q2)*q2);}
