#ifndef ERS_ERRH
#define ERS_ERRH 1
/* C language ERROR include file
 * Generated by /Users/drama/release/messgen/r3_16/macosx_x86_64/messgen utility
 * from input file ../Ers_Err.msg at Tue Dec  8 19:01:48 2020
 */

/* Facility number */
#define ERS__FACNUM 1827

/* OK Status symbol */
#define ERS__OK 0

/* The ERS system error table is full */
#define ERS__NOSPACE 0xF238012

/* The ERS system is already active */
#define ERS__ACTIVE 0xF23801A

/* The ERS system is not active */
#define ERS__NOTACTIVE 0xF238022

/* ErsPop was called more times then ErsPush */
#define ERS__TOOMANPOPS 0xF23802A

/* Error adding or deleting task a variable */
#define ERS__TASKVARERR 0xF238032

/* Malloc error */
#define ERS__MALLOCERR 0xF23803A

/* Ers already initialised */
#define ERS__INITIALISED 0xF238042

/* Ers not initialised */
#define ERS__NOTINITIALISED 0xF23804A

/* Operation not permissible while flushing */
#define ERS__FLUSHING 0xF238052

/* Operation (ErsClear/ErsFlush) is not permissible while ERS is in interrupt mode */
#define ERS__INTHAND 0xF23805A

#endif
