#ifndef TDFXY_ERR_MSGTH
#define TDFXY_ERR_MSGTH 1
/*  Message table include file
 *  Generated by /Users/drama/release/messgen/r3_16/macosx_x86_64/messgen utility
 *  from input file /Users/ks/acmm/2dFutil/tdfxy_err.msg at Thu Dec 10 10:44:58 2020
  */
static MessTabType MessFacTab_TDFXY[] = {
    { 100, "ILLRA", 
        "Illegal RA" },
    { 101, "ILLDEC", 
        "Illegal Dec" },
    { 102, "ZDERR", 
        "ZD greater than 70 degrees" },
    { 103, "SPRINTF", 
        "Format string exceeds available string length (ErsSPrintf)" },
    { 104, "DIRCREERR", 
        "Error creating directory" },
    { 105, "SLA_ERR_1", 
        "slaDs2tp failed - star too far from Axis." },
    { 106, "SLA_ERR_2", 
        "slaDs2tp failed - Antistar on tanget plane" },
    { 107, "SLA_ERR_3", 
        "slaDs2tp failed - Antistar to far from axis" },
    { 108, "SLA_ERR_UNKN", 
        "slaDs2tp failed - Unexpected error code" },
    { 109, "ILLMJD", 
        "Illegal Mean Julian Date (MJD) specified - object cannot be observed" },
    { 110, "ILLTEMP", 
        "Illegal temperature supplied to cvtInit routine - outside expected range" },
    { 111, "ILLPRES", 
        "Illegal pressure supplied to cvtInit routine - outside expected range" },
    { 112, "ILLHUMI", 
        "Illegal humidity supplied to cvtInit routine - outside expected range" },
    { 113, "ILLWAVE", 
        "Illegal wavelength supplied to cvtInit routine - outside expected range" },
    { 114, "ILLSCALE", 
        "Illegal scale - cannot have a scale of zero" },
    { 115, "ILLFIELD", 
        "Illegal field plate number - must be 0,1 or 2" },
    { 116, "MAP_FILE_OPEN", 
        "Failed to open FPI to Sky distortion map file" },
    { 117, "MAP_FILE_READ", 
        "Failed to read FPI to Sky distortion map file" },
    {0, 0, 0 }};

static MessFacType MessFac_TDFXY = 
  { 1851, "TDFXY", MessFacTab_TDFXY, 0 };

#endif
